/*******************************************************************************
 *     ___                  _   ____  ____
 *    / _ \ _   _  ___  ___| |_|  _ \| __ )
 *   | | | | | | |/ _ \/ __| __| | | |  _ \
 *   | |_| | |_| |  __/\__ \ |_| |_| | |_) |
 *    \__\_\\__,_|\___||___/\__|____/|____/
 *
 *  Copyright (c) 2014-2019 Appsicle
 *  Copyright (c) 2019-2022 QuestDB
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 ******************************************************************************/

package io.questdb.log;

import io.questdb.std.Numbers;

public final class LogLevel {
    public static final int DEBUG = 1;
    public static final int INFO = 2;
    public static final int ERROR = 4;
    public static final int CRITICAL = 8;
    public static final int ADVISORY = 16;
    public static final int ALL = DEBUG | INFO | ERROR | CRITICAL | ADVISORY;
    public static final int MAX = Numbers.msb(LogLevel.ADVISORY) + 1;
    public static final int MASK = ~(-1 << (MAX));

    public static final String DEBUG_HEADER = " D ";
    public static final String INFO_HEADER = " I ";
    public static final String ERROR_HEADER = " E ";
    public static final String CRITICAL_HEADER = " C ";
    public static final String ADVISORY_HEADER = " A ";

    private LogLevel() {
    }
}
